/*
	HEADER:		CUG236;
	TITLE:		Entab/Detab/Strip Combo -- Filter Version;
	DATE:		05/25/1987;
	DESCRIPTION:	"Strips upper-order bits, detabifies, then entabifies a
			text file.  Old and new tab increments can differ.  If
			new increment is 1, no entabification occurs.";
	VERSION:	0.0;
	KEYWORDS:	Tab, Tabs, Tab Expansion, Tab Utility, Tabify, Untab,
			Detab, Detab Utility;
	FILENAME:	RETABF.C;
	COMPILERS:	vanilla;
	AUTHORS:	W. C. Colley III;
*/

/*
			 Tab Size Readjustment Program

		   Copyright (c) 1987 William C. Colley, III

This program combines the functions of an ENTAB program and a DETAB program
in the same program.  Combining the two allows a simple means of retabifying
a program.  For example, let's say you edit a C program with your editor set
for a tab increment of 4.  Now, you TYPE this file on the console and you find
that it laps over the right edge of the screen because your terminal expands
tabs with a tab increment of 8.	 What you really would like to do is delete
some of the tabs so that your program will TYPE just like it looked inside the
editor.	 The following command will do just that:

	RETABF -O4 -N8 <oldfile >newfile

In the above command line, the parameters can appear in any order.  The -
options are optional.  Their default values are set by a pair of #defines
below.	The -O option specifies the tab increment that the old file was
created with.  The -N option specifies the desired tab increment of the new
file.

Note that or -N1 causes no entabification.  Thus, the following command will
detabify a file:

	RETABF -O8 -N1 <oldfile >newfile

Also note that if the old file contains no tabs, there will be no
detabification.	 Thus, the following command will entabify a detabified file:

	RETAB -N8 <oldfile >newfile

Blanks and tabs in quoted strings are left alone.  The C conventions regarding
character constants like '"' and character strings like "Here\'s a \"quote\"."
are supported.
*/

#include <stdio.h>

/*  User-adjustable default values:					    */

#define OLDTABS	    4	    /*	Default old tab increment.		    */
#define NEWTABS	    8	    /*	Default new tab increment.		    */

#define NORMAL	    0	    /*	Normal state.				    */
#define QUOTE	    1	    /*	Quoted string skip state.		    */
#define BLANK	    2	    /*	Blank parsing state.			    */

int main(argc,argv)
int argc;
char **argv;
{
    int del, i, j, atoi();
    static int err = 0, oldtabs = OLDTABS, newtabs = NEWTABS;
    static int blnks = 0, col = 0, lit = 0, state = NORMAL;

    while (--argc) {
	if (**++argv == '-') {
	   r > 12 )	/* Adjust 24 hr. to 12 hr. time	*/
		{
			hour -= 12;
			pm = 1;
		}

		sprintf(buf, "%02d:%02d %s", hour, regs.h.cl, pm? "PM":"AM");
	}

	return( buf );
}

/*----------------------------------------------------------------------*/

LOCAL char	*gdate()
{
	/* Return a pointer to a string that holds the current date.
	*/

	union   REGS	regs;
	static	char	buf[14];

	static char	*weekdays[] =
	{
		"Sun",	"Mon",	"Tue",	"Wed",	"Thu",	"Fri",	"Sat"
	};

	regs.h.ah = 0x2a ;
	intdos( &regs, &regs );

	sprintf(buf, "%s %02d-%02d-%02d", weekdays[ regs.h.al ],
				regs.h.dh, regs.h.dl, regs.x.cx - 1900 );
	return( buf );
}

/*----------------------------------------------------------------------*/


char	*scan( str, one_of_these, strip, nwild, nchars )
char	*str, *one_of_these;
int	strip, *nwild, *nchars ;
{
	/* Find the first character in str that is also in one_of_these.
	 * Characters inside quoted strings and escaped
	 * characters are ignored. Return a pointer into str at the
	 * matched character or NULL if no match is found.
	 * This routine can't be used to find a quote mark. If strip
	 * is true, non-escaped ' characters and Esc characters are deleted.
	 * *nwild is incremented every time a non-escaped wildcard is
	 * found. *nchars is decremented every time a character is
	 * deleted.
	 */

	register char	*p;
	register int	quote = 0;	/* 0 if not in a quoted string else */
					/* holds the kind of quote we're    */
					/* processing ( ' or "  ).	    */

	while( *str )
	{
		if( *str == Esc && (!quote || ISQUOTE(str[1])) )
		{
			/* Escaped characters not in quoted strings
			 * or escaped quotes that are in quoted strings
			 */

			if(strip) { (*nchars) -= deletes(1, str); }
			else	  { str++;			  }

			if( *str )
				str++;
		}
		else if( (!quote && ISQUOTE(*str)) || (quote && *str==quote) )
		{
			/* non-escaped quote characters (only ' will
			 * be stripped).
			 */

			quote = quote ? 0 : *str ;

			if( strip && *str == '\'' )
				(*nchars) -= deletes(1, str);
			else
				str++;
		}
		else
		{				/* Normal characters */
			if( !quote )
			{
				for( p = one_of_these; *p && *p != *str; p++ )
					;

				if( *p )
					return str;

				if( ISWILD(*str) )
					(*nwild)++;
			}

			str++;
		}
	}

	return