#define MSDOS2

#include "dos.h"
#include "stdio.h"
#include "ctype.h"
#include "ios1.h"

#define MAXARG 100              /* maximum command line arguments */

extern int _stack,_fmode,_iomode;
extern char _iname[],_oname[],_dos;
extern struct UFB _ufbs[];

static int argc;			/* arg count */
static char *argv[MAXARG];		/* arg pointers */


/**
 *
 * name         _main - process command line, open files, and call "main"
 *
 * synopsis     _main(line);
 *              char *line;     ptr to command line that caused execution
 *
 * description	This function performs the standard pre-processing for
 *		the main module of a C program.  It accepts a command
 *		line of the form
 *
 *			pgmname arg1 arg2 ...
 *
 *		and builds a list of pointers to each argument.  The first
 *		pointer is to the program name.  For some environments, the
 *		standard I/O files are also opened, using file names that
 *		were set up by the OS interface module XCMAIN.
 *
 ********
 *		Modified: 11 April 1984
 *			  Ross Nelson
 *
 *		This routine now treats quoted strings in the command line
 *		as a single token, and allows the expansion of wildcard
 *		characters (see WILDCARD.C).  A quoted string begins with
 *		one of the characters ' or " and continues until that
 *		character is found again (unless escaped by a \).  Any
 *		command argument containing the MSDOS wildcard characters
 *		(* and ?) must be quoted or this routine will attempt to
 *		perform filename expansion on them.  The program will
 *		exit if any wildcard expansion fails to match.	If wildcard
 *		expansion fills available memory or overflows MAXARG, a
 *		messaage will be printed, but processing will continue.
 ********
 *
 *		Further mods: Oct-11-84 Chuck Forsberg V2.12, Unix pathnames
 *
 */
_main(line)
char *line;
{
	char c;
	char quoted;
	int x;
#ifdef MSDOS2
	char buf[135], *wild, *malloc();
#endif
#if UNIX == 0
	FILE *fp0, *fp1, *fp2;
	extern int _bufsiz;
	char *getmem();
#endif

/*
 *
 * Build argument pointer list
 *
 */
	for(argc = 0; argc < MAXARG; )
	{
		while(isspace(*line))
			line++;
		if(*line == '\0')
			break;
		argv[argc++] = line;
		quoted = 0;
		if (((*line == '\'') || (*line == '"')) && (*(line-1) != '\\')) 
		{
			quoted++;
			c = *line;
			argv[argc-1]++;
			while (*++line)
				if ((*line == c) && (*(line-1) != '\\'))
					break;
		}
		else
			while((*line != '\0') && (isspace(*line) == 0))
				line++;
		c = *line;
		*line++ = '\0';
#ifdef MSDOS2
		if (!quoted && stpbrk(argv[argc-1], "*?")) {
			wild = argv[--argc];
			if (!wildcard(wild, 0x10, bufmeters
		 * to a termporary buffer.
		 */

		i = c - str;
		if (parmstr = malloc(i + 10 /*headroom*/) )
		{
		    strncpy(parmstr, str, i);
		    parmstr[i] = '\0';
		}
		else
		{
		    syserr( "sh: Not enough memory for alias parameters\n" );
		    if (*c)
		    {
			str = c + 1;
			oldstr = "";
		    }
		    continue;
		}

		/* Delete the actual parameters from command line */

		deletes(i, str);

		/* Break the parameters up and build parm[], an argv-like
		 * list of paramters
		 */

		for( c = parmstr, i = 0; *c && i < 10; ++i )
		{
		    if( (i > maxparam) )
		    {
			tail = c;
			break;
		    }

		    parm[i] = c;
		    if (ISQUOTE(*c))
		    {
			/* Keep quoted parameters as a single parm */
			chr = *c++;
			do
			{
			    if( *c == Esc && *++c )
				++c;
			} while (*c && *c++ != chr);
		    }
		    else
		    {
			/* Find whitespace or '\0' delimiter to parm */

			while( *c && !isspace(*c) )
			    ++c;
		    }
		    if (*c)
		    {
			/* If parameter wasn't whitespace delimited
			 * (quoted parameter or parm followed by '/0')
			 * insert a space and then make it a '\0'.
			 * this is where we might need the headroom.
			 */

			if( !isspace(*c) )
			    inserts(" ", c, strlen(c) + 2);

			/* Terminate parm with '\0' and skip to next */

			*c++ = '\0';
			SKIPWHITE(c);
		    }
		}

		/* Set unused parameters to empty string */

		while( i < 10 )
		    parm[i++] = "";
    
#		ifdef DEBUG
		    for (i = 0; *parm[i]; ++i)
			printf("@%d: %s\n", i, parm[i]);
#		endif

		/* Copy the alias body into the command line buffer,
		 * substituting parameters as necessary.
		 */

		en