/* *************************************************************** */
/* This program reads a series of words from the command line,     */
/* and displays all but the last on the monitor. The last is a     */
/* series of characters which are used as input comparisons. One   */
/* character is read from the keyboard. If it is one of the        */
/* characters in the comparison list, its number is returned to    */
/* DOS as the errorlevel command. If the character does not exist  */
/* in the list, a zero is returned. Example follows;               */
/*                                                                 */
/* WHATNEXT What model do you want? ALR%3T                         */
/*                                                                 */
/* What model do you want?      <---- displayed on monitor         */
/*    If key a or A is hit, errorlevel 1 is returned.              */
/*    If key l or L is hit, errorlevel 2 is returned.              */
/*    If key r or R is hit, errorlevel 3 is returned.              */
/*    If key % is hit, errorlevel 4 is returned.                   */
/*    If key 3 is hit, errorlevel 5 is returned.                   */
/*    If key t or T is hit, errorlevel 6 is returned.              */
/*    If any other key is hit, errorlevel 0 is returned.           */
/*                                                                 */
/*  The question must be on one line.                              */
/*  Up to nine different keys can be used.                         */
/*  The errorlevel can be interpreted in a batchfile.              */
/* *************************************************************** */

#include "stdio.h"
#include "ctype.h"

main(number,name)
int number;               /* total number of words on command line */
char *name[];
{
int index;                  /* a counter and incrementing variable */
int c;                     /* the character read in for comparison */
int code;                  /* the resulting errorlevel returned to */
char next_char;                    /* used for the comparison loop */
char *point;               /* a dummy pointer used for convenience */

                     /* At least one group must be used for this   */
                     /* filename, and one group used for the       */
                     /* required fields, so less than three allows */
                     /* for no question.                           */
   if (number < 3) {
      printf("No question given on command line\n");
      exit(0);
   }

          stall interrupt handler

; save dseg for interrupt handler
	mov		ax,ds
	mov		cs:dssave,ax

; set int vector to my handler
	push	ds
	xor		ax,ax
	mov		ds,ax
	mov		[30h],offset handler
	mov		ax,cs
	mov		[32h],ax
	pop		ds

; assert DTR, RTS, and OUT2
	mov		dx,3fch
	mov		al,11
	out		dx,al

; enable interrupts for data ready
	mov		dx,3f9h
	mov		al,1
	out		dx,al

; clear any outstanding int
	mov		dx,3f8h
	in		al,dx

; enable interrupts
	sti

; enable IRQ4 on 8259A
	in		al,21h
	and		al,0efh
	out		21h,al

; and return
	jmp		init_finis

dssave:	dw 0

handler:
	push	ds
	push	ax
	push	dx
	push	bx

; restore appropriate dseg
	mov		ax,cs:dssave
	mov		ds,ax

; grab the byte from comm port
	mov		dx,3f8h
	in		al,dx

; put it in the buffer
	mov		bx,tail_
	mov		comm_buffer_[bx],al

; tail=tail+1 (mod 8192)
	inc		bx
	cmp		bx,8192
	jl		handler_around
	xor		bx,bx

handler_around:
	mov		tail_,bx

; tell the 8259A EOI
	mov		dx,20h
	mov		al,32
	out		dx,al

	pop		