/*	module:		window.c
 *	programmer:	Ray L. McVay
 *	started:	1 Aug 84
 *	version:	1.2, 23 Aug 84
 *
 *	A simple window package based on the article and programs
 *	by Edward Mitchell in the Jan 84 Dr. Dobb's Journal.  This
 *	implementation uses dynamically allocated buffers for the
 *	window control blocks and screen save buffers.
 *
 *	An assembly language support library called VLIB is used to
 *	interface the DeSmet C compiler with the IBM ROM BIOS video
 *	routines.  VLIB will undoubtedly have to be rewritten if the
 *	window package is to be used with other compilers.
 *
 *	History -
 *
 *	1.1	-	Added style member to wcb structure, clr_window() and
 *			use of wn->cx for horizontal scrolling in write_text().
 *
 *	1.2 -	Added oldx, oldy members to wcb structure and use them
 *			in draw_window() and remove_window().
 *
 */

#include	<stdio.h>

typedef struct wcb	/* --- window control block	---		*/
{
	int		ulx,	/* upper left corner x coordinate	*/
			uly,	/* ditto y coordinate				*/
			xsize,	/* line width of inside of window	*/
			ysize,	/* number of lines inside of window	*/
			cx,		/* current cursor offset in window	*/
			cy,
			style,	/* attribute to be used in window	*/
			*scrnsave,	/* pointer to screen save buffer */
			oldx,	/* cursor position when window was	*/
			oldy;	/* opened (used for screen restore)	*/
} WINDOW, *WINDOWPTR;


/************************************************************************
* putch(), write a character and attribute to a specific XY location on *
*		   the screen. The attribute is the high byte of the character.	*
************************************************************************/

putch(x, y, c)
int		x, y, c;
{
	gotoxy(x, y, 0);
	vputca(c, 0, 1);
}


/************************************************************************
* getch(), return the character and attribute at screen XY				*
************************************************************************/

getch(x, y)
int		x, y;
{
	gotoxy(x, y, 0);
	return(vgetc(0));
}


/************************************************************************
* draw_row(), output a row of one character/attribute at XY				*
************************************************************************/

draw_row(x, y, count, c)
int		x, y, count,c;
{
	gotoxy(x, y, 0);
	vputca(c, 0, count);
}


/************************************************************************
* draw_window(), open a window of given size with upper left corner at	*
*				 XY. Allocates buffers for the window control block and *
*				 screen save buffers. Copies o sgr ("1;4;5;0;7");
    printf ("negative");
    cup (12, 40);
    sgr ("0;1;7");
    printf ("bold negative");
    cup (14, 6);
    sgr ("0;4;7");
    printf ("underline negative");
    cup (14, 45);
    sgr ("0;1;4;7");
    printf ("bold underline negative");
    cup (16, 1);
    sgr ("1;4;;5;7");
    printf ("blink negative");
    cup (16, 40);
    sgr ("0;1;5;7");
    printf ("bold blink negative");
    cup (18, 6);
    sgr ("0;4;5;7");
    printf ("underline blink negative");
    cup (18, 45);
    sgr ("0;1;4;5;7");
    printf ("bold underline blink negative");
    sgr ("");

    rm ("?5");						/* Inverse video off */
    cup (23, 1);
    el (0);
    getlin ("Black background. Push <RETURN>");
    sm ("?5");						/* Inverse video */
    cup (23, 1);
    el (0);
    getlin ("Inverse video. Push <RETURN>");
    rm ("?5");
    ed (2);
    cup (8, 12);
    printf ("normal");
    cup (8, 24);
    printf ("bold");
    cup (8, 36);
    printf ("underscored");
    cup (8, 48);
    printf ("blinking");
    cup (8, 60);
    printf ("reversed");
    cup (10, 1);
    printf ("stars:");
    cup (12, 1);
    printf ("line:");
    cup (14, 1);
    printf ("x'es:");
    cup (16, 1);
    printf ("diamonds:");
    for (cset = 0; cset <= 3; cset++) {
	for (i = 0; i <= 4; i++) {
	    cup (10 + 2 * cset, 12 + 12 * i);
	    sgr (attr[i]);
	    if (cset == 0 || cset == 2)
		scs (0, 'B');
	    else
		scs (0, '0');
	    for (j = 0; j <= 4; j++) {
		printf ("%c", tststr[cset]);
	    }
	    pcsc ();
	    cup (cset + 1, i + 1);
	    sgr ("");
	    scs (0, 'B');
	    printf ("A");
	    pcrc ();
	    for (j = 0; j <= 4; j++) {
		printf ("%c", tststr[cset]);
	    }
	}
    }
    sgr ("0");
    scs (0, 'B');
    cup (22, 1);
    printf ("There should be 10 characters of each flavour, and a rectangle\n");
    getlin ("of 5 x 4 A's filling the top left of the screen. Push <RETURN>");
}

tst_doublesize () {
							/* Test of: PCSWL  (Single Width Line) PCDWL  (Double Width
							   Line) PCDHL  (Double Height Line) (also implicit double
							   width) */

    int     i, w, w1;

 /* Print the test pattern in both 80 and 132 character width  */

    for (w = 0; w <= 1; w++) {
	w1 = 13 * w;

	ed (2);
	cup (1, 1);
	if (w) {
	    sm ("?3");
	    printf ("132 column mode");
	}
	else {
	    rm ("?3");
	    printf (" 80 column mode");
	}

	cup (5, 3 + 2 * w1);
	printf ("v------- left margin");

	cup (7, 3 + 2 * w1);
	printf ("This is a normal-sized line");
	pcdhl (0);
	pcdhl (1);
	pcdwl ();
	pcswl ();

	cup (9, 2 + w1);
	printf ("This is a Double-width line");
	pcswl ();
	pcdhl (0);
	pcdhl (1);
	pcdwl ();

	cup (11, 2 + w1);
	pcdwl ();
	pcswl ();
	pcdhl (1);
	pcdhl (0);
	printf ("This is a Double-width-and-height line");
	cup (12, 2 + w1);
	pcdwl ();
	pcswl ();
	pcdhl (0);
	pcdhl (1);
	printf ("This is a Double-width-and-height line");

	cup (14, 2 + w1);
	pcdwl ();
	pcswl ();
	pcdhl (1);
	pcdhl (0);
	el (2);
	printf ("This is another such line");
	cup (15, 2 + w1);
	pcdwl ();
	pcswl ();
	pcdhl (0);
	pcdhl (1);
	printf ("This is another such line");

	cup (17, 3 + 2 * w1);
	printf ("^------- left margin");

	cup (21, 1);
	printf ("This is not a double-width line");
	for (i = 0; i <= 1; i++) {
	    cup (21, 6);
	    if (i) {
		printf ("**is**");
		pcdwl ();
	    }
	    else {
		printf ("is not");
		pcswl ();
	    }
	    cup (23, 1);
	    getlin ("Push <RETURN>");
	}
    }

    rm ("?3");
    ed (2);

    scs (0, '0');
    cup (8, 1);
    pcdhl (0);
    printf ("lqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqk");
    cup (9, 1);
    pcdhl (1);
    printf ("lqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqk");
    cup (10, 1);
    pcdhl (0);
    printf ("x                                   x");
    cup (11, 1);
    pcdhl (1);
    printf ("x                                   x");
    cup (12, 1);
    pcdhl (0);
    printf ("x                                   x");
    cup (13, 1);
    pcdhl (1);
    printf ("x                                   x");
    cup (14, 1);
    pcdhl (0);
    printf ("x                                   x");
    cup (15, 1);
    pcdhl (1);
    printf ("x    verwritten screen to the	*
*				 buffer. Draws the blank window. Returns the address of	*
*				 the window control block or NULL if no buffer space.	*
************************************************************************/

WINDOWPTR draw_window(x, y, width, height, attrib)
int		x, y, width, height, attrib;
{
	WINDOWPTR wn;
	int		tx, ty,
			xend, yend;
	int		*tptr;
	char	*calloc();

	if ((wn = (WINDOWPTR)calloc(1, sizeof(WINDOW))) == NULL)
		return(NULL);
	else if ((wn->scrnsave = (int *)calloc((width+2) * (height+2), sizeof(int))) == NULL)
	{
		free(wn);
		return(NULL);
	}
	else
	{
		/* store parameters in window control block */

		wn->ulx = x;
		wn->uly = y;
		wn->xsize = width;
		wn->ysize = height;
		wn->cx = 1;
		wn->cy = 1;
		wn->style = attrib;
		attrib <<= 8;	/* will make things below go quicker */
		wn->oldx = getxy(0) & 255;
		wn->oldy = getxy(0) >> 8;

		/* Copy existing text where the window will be placed */
		/* to the scrnsave buffer. Obviously, a less portable */
		/* routine could be much faster.					  */

		tptr = wn->scrnsave;
		xend = x + width + 2;
		yend = y + height + 2;

		for (ty = y; ty < yend; ty++)
		{
			for (tx = x; tx < xend; tx++)
				*tptr++ = getch(tx, ty);
		}

		/* draw the window border and clear the text area */

		putch(x, y, 0xda + attrib);					/* ul corner */
		draw_row(x + 1, y, width, 0xc4 + attrib);	/* horiz bar */
		putch(x + width + 1, y, 0xbf + attrib);		/* ur corner */

		yend = y + height;

		for (ty = y+1; ty <= yend; ty++)
		{
			putch(x, ty, 0xb3 + attrib);			/* draw the sides */
			putch(x+width+1, ty, 0xb3 + attrib);
		}

		putch(x, y + height + 1, 0xc0 + attrib);				/* ll corner */
		draw_row(x + 1, y + height + 1, width, 0xc4 + attrib);	/* horiz bar */
	